
#include "libgfx.c"

#include "builtin.hpp"
#include "libgfx.hpp"

namespace __libgfx__ {


str *__name__;


// this stuff uses a table to translate pointers to canvases into 
// ints - this is only needed if you are running in 64 bits mode.  
// If you run python in 32 bit mode, this can just return
// the pointer to python.

#define TABSIZE	500
static int curpos = 0;
static canvas *tab[500];

__ss_int canptr2handle(canvas *c)
{
    for(int i=0; i<curpos; i++) { 	// look for it in the table 1 added so we never return 0
	if(tab[i] == c)
	    return (__ss_int)(i+1);
    }
    for(int i=0; i<curpos; i++) {	// if not found, look for an empty space
	if(tab[i] == 0) {
    	    tab[i] = c;
	    return (__ss_int)(i+1);
	}
    }
    tab[curpos] = c;			// if no empty space use the next space.
    curpos++;
    if(curpos>TABSIZE) {
	fprintf(stderr, "handle table is to small\n");
	exit(1);
    }
    return (__ss_int)(curpos);
}

canvas *handle2canptr(__ss_int i)
{
    return tab[i-1];
}

void handlefree(canvas *c)
{
    tab[canptr2handle(c)-1] = 0;
}


__ss_int gfx_cannew(__ss_int sizex, __ss_int sizey) {
    return canptr2handle(cannew(sizex, sizey));
}

void *gfx_canfree(__ss_int ic) {
    canvas *c = handle2canptr(ic);
    canfree(c);
    handlefree(c);
    return NULL;
}

__ss_int gfx_cannalloc() {
    return (__ss_int)cannalloc();
}

__ss_int gfx_canclone(__ss_int ic) {
    canvas *c = handle2canptr(ic);
    return canptr2handle(canclone(c));
}

list<__ss_int> *gfx_cansize(__ss_int ic) {
    canvas *c = handle2canptr(ic);
    list<__ss_int> *_list;
    _list = (new list<__ss_int>());
    _list->append(c->sizex);
    _list->append(c->sizey);
    _list->append(4);
    return _list;
}

void *gfx_cansaturate(__ss_int ic, double sat) {
    canvas *c = handle2canptr(ic);
    cansaturate(c, sat);
    return NULL;
}

void *gfx_canclear(__ss_int ic, double r, double g, double b, double a) {
    canvas *c = handle2canptr(ic);
    canclear(c, r, g, b, a);
    return NULL;
}

void *gfx_canscalergba(__ss_int ic, double r, double g, double b, double a) {
    canvas *c = handle2canptr(ic);
    canscalergba(c, r, g, b, a);
    return NULL;
}

__ss_int gfx_cangetpix(__ss_int ic, __ss_int x, __ss_int y) {
    canvas *c = handle2canptr(ic);
    return (__ss_int)cangetpix(c, x, y);
}

void *gfx_canputpix(__ss_int ic, __ss_int x, __ss_int y, __ss_int pix) {
    canvas *c = handle2canptr(ic);
    canputpix(c, x, y, pix);
    return NULL;
}

__ss_int gfx_canshift(__ss_int ic, __ss_int shiftx, __ss_int shifty) {
    canvas *c = handle2canptr(ic);
    return canptr2handle(canshift(c, shiftx, shifty));
}

__ss_int gfx_canframe(__ss_int ic, double r, double g, double b, double a, double width) {
    canvas *c = handle2canptr(ic);
    return canptr2handle(canframe(c, r, g, b, a, (int)width));
}

void *gfx_canmult(__ss_int ic1, __ss_int ic2) {
    canvas *c1 = handle2canptr(ic1);
    canvas *c2 = handle2canptr(ic2);
    canmult(c1, c2);
    return NULL;
}

void *gfx_canblend(__ss_int ic1, __ss_int ic2, double r, double g, double b, double a) {
    canvas *c1 = handle2canptr(ic1);
    canvas *c2 = handle2canptr(ic2);
    canblend(c1, c2, a, g, b, a);
    return NULL;
}

__ss_int gfx_cansubimg(__ss_int ic, double orgx, double orgy, double sizex, double sizey) {
    canvas *c = handle2canptr(ic);
    return canptr2handle(cansubimg(c, orgx, orgy, sizex, sizey));
}

__ss_int gfx_canzoom(__ss_int ic, double zoomx, double zoomy) {
    canvas *c = handle2canptr(ic);
    return canptr2handle(canzoom(c, zoomx, zoomy));
}

__ss_int gfx_canfromfile(str *filename) {
    return canptr2handle(canfromfile(filename->unit.c_str()));
}

void *gfx_cantofile(__ss_int ic, str *filename) {
    canvas *c = handle2canptr(ic);
    cantofile(c, filename->unit.c_str());
    return NULL;
}

void __init() {
    __name__ = new str("libgfx");

}

} // module namespace
